class Vote < ActiveRecord::Base
  # relationships to other classes
  belongs_to :voter
  has_many :rankings
  has_one :token
  
  # callbacks
  after_update :save_rankings
  before_destroy :destroy_rankings

  def to_s
    votes.join("")
  end

  def each
    votes.each {|vote| yield vote}
  end

  def votes
    unless @votes
      if rankings.empty?
        @votes = Array.new
      else
        @votes = rankings.sort.collect { |ranking| ranking.candidate.id }
      end
    end

    @votes
  end

  def votes=(array)
    @votes = array
  end

  def save_rankings
    destroy_rankings
    self.votes.each_with_index do |candidate, index| 
      ranking = Ranking.new
      ranking.rank = index + 1
      ranking.candidate =  Candidate.find(candidate)
      self.rankings << ranking
    end
  end

  def destroy_rankings 
    rankings.each { |ranking| ranking.destroy }
  end

  def confirm!
    self.confirmed = 1
    self.save

    token.destroy and token.reload if token
    self.token = Token.new
    self.save
  end

  def confirm?
    if confirm == 1
      return true
    else 
      return false
    end
  end
  
  def votestring=(string="")
    candidate_ids = voter.election.candidates.sort.collect \
      { |candidate| candidate.id.to_i }

    rel_votes = string.split("").collect { |vote| vote.to_i }
    
    # covert relative orders to absolute candidate ids
    self.votes = rel_votes.collect { |vote| candidate_ids[ vote - 1 ] }
  end

end
